/*
 * rocket.scad -- rocket functions
 *
 * (c) 2014 Prof Dr Andreas Mueller
 */
include <parameters.scad>
$fn = 30;


finradius = finspan + radius;

module fin() {
	translate([finradius/2, 0, 40])
		cube([finradius + 1, finthickness, 80], center = true);
	translate([radius - 1, 0, 40])
		rotate([0, 0, 45])
			cube([3 * finthickness + sqrt(2), 3 * finthickness + sqrt(2), 80],
				center = true);
}

module fincan() {
	intersection() {
		union() {
			rotate([0, 0, 0]) fin();
			rotate([0, 0, 120]) fin();
			rotate([0, 0, 240]) fin();
		}
		union() {
			translate([0, 0, 50])
				cylinder(r1 = finradius, r2 = 0, h = 30);
			cylinder(r = finradius, h = 50);
		}
	}
}

module launchlug() {
	intersection() {
		union() {
			translate([0, 0, 10])
				cylinder(r1 = radius + 10, r2 = 0, h = radius + 10);
			translate([0, 0, -20])
				cylinder(r1 = 0, r2 = radius + 10, h = radius + 10);
		}
		translate([-radius - 2.5, 0, 0])
			difference() {
				cylinder(r = 4, h = radius + 10);
				translate([0, 0, -1])
					cylinder(r = 2, h = radius + 12, $fn = 20);
			}
	}
};


module anchor() {
	intersection() {
		translate([0, radius - wallthickness - 1.5 - 0.75, 5])
			cube([2 * radius, 1.5, 10], center = true);
		cylinder(r = radius - wallthickness + 0.2, h = 10);
	}
}

module airframe() {
	union() {
		difference() {
			union() {
				cylinder(r = radius, h = height, $fn = 60);
				fincan();
				translate([0, 0, 275]) launchlug();
				translate([0, 0, 0]) launchlug();
			}
			union() {
				// hole for the motor
				translate([0, 0, -1]) cylinder(r = 9, h = 71);
				// tapered hole used as thrust ring
				translate([0, 0, 70]) cylinder(r1 = 9, r2 = 7, h = 2);
				// opening for ejection gases
				cylinder(r = 7, h = 90);
				// tapered hole for parachute compartement
				translate([0, 0, 75])
					cylinder(r1 = 7, r2 = radius - wallthickness, h = 10);
				// parachute compartment
				translate([0, 0, 85])
					cylinder(r = radius - wallthickness, h = height, $fn = 60);
				// opening for the motor retainer clip
				translate([hookoffset + hookheight / 2, 0, 35 + 1 + 1.5 + 1])
					cube([hookheight, hookwidth, 70], center = true);
				translate([hookoffset + hookheight / 2, 0, 0.75 + 1])
					cube([hookheight, hookwidth, 1.5], center = true);
				translate([hookoffset + (hookheight - 1) / 2, 0, -1])
					cube([hookheight - 1, hookwidth, 20], center = true);
			}
		}
		translate([0, 0, height - 2 * radius]) rotate([0, 0, 90])
			anchor();
	}
};

// Haack Sears nose cone
// Octave code:
/*
for i = (0:20)
	x = (20 - i) / 40;
	y = (4 * x * (1 - x))^(3/4);
	printf("\t[%f, %f],\n", y, 2 *x);
endfor
*/

Rmax = radius;
L = noseconeheight;

module noseshape() {
    rotate([180, 0, 0])
	translate([0, 0, -L])
		rotate_extrude(convexity = 2)
			scale([Rmax, L])
				polygon(points = [
					[1.000000, 1.000000],
					[0.998124, 0.950000],
					[0.992491, 0.900000],
					[0.983077, 0.850000],
					[0.969847, 0.800000],
					[0.952749, 0.750000],
					[0.931711, 0.700000],
					[0.906641, 0.650000],
					[0.877424, 0.600000],
					[0.843914, 0.550000],
					[0.805927, 0.500000],
					[0.763235, 0.450000],
					[0.715542, 0.400000],
					[0.662467, 0.350000],
					[0.603500, 0.300000],
					[0.537940, 0.250000],
					[0.464758, 0.200000],
					[0.382338, 0.150000],
					[0.287783, 0.100000],
					[0.174483, 0.050000],
					[0.000000, 0.000001],
					[0.000000, 1.000000]
				]);
}

noseconescale = (radius - wallthickness) / radius;

module nosecone() {
	union() {
		difference() {
			union() {
				noseshape();
				translate([0, 0, -radius / 2])
					cylinder(r = radius - wallthickness - gap, h = radius, center = true,
						$fn = 60);
				translate([0, 0, -0.99])
					cylinder(r2 = radius, r1 = radius - 2, h = 2, center = true, $fn = 60);
			}
			union() {
				scale(noseconescale * [1, 1, 1])
						noseshape();
				translate([0, 0, -radius]) sphere(r = 8, center = true);
				for (phi = [0: 60: 359]) {
					rotate([0, 0, phi])
						translate([0, (radius - wallthickness - gap + 8) / 2, -radius / 2 - 0.5])
							cylinder(r = 3, h = radius + 2, center = true);
				}
				translate([0, 0, -radius + 10])
					cylinder(r = radius - 2 * wallthickness, h = radius - 8 + wallthickness);
			}
		}
		translate([0, 0, -radius + 2])
			cube([2 * (radius - wallthickness - 2), 4, 4], center=true);
	}
};

delta = tension + hookheight;

module hookshape() {
	polygon(points = [
			[2,-1], [10, -1], [10, -4], [11, -4], [20, 5], [19, 6], [13, 2], [3, 2], [-1, 6],
			[-1, hookheight],
			[-2, hookheight], [-2, hookheight - 1.5], [-4, hookheight - 1.5], [-4, hookheight], 
			[-15, hookheight],
			[-60, -tension], [-60, -tension - 1],
			[-30, (hookheight + tension) * 2/3 - tension - 1],
			//[-10, hookheight - 5],
			[-14, hookheight - 2],
			[-7, hookheight - 5],
			[-2, hookheight - 5]
		]);
}

module hook() {
	linear_extrude(height = hookwidth - 0.5) {
		hookshape();
	}
}
